/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                           BIAYSTOK 2017                               **
**                                                                        **
**                         Home Control System                            **
**                                                                        **
**                           TimeEventModule                              **
**                      Wersja z dnia : 2018.09.01                        **
**                     Napisa : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/


#include "timeeventmodule.h"
#include <inttypes.h>

#define FreeElementStatus              0
#define SingleCallServiceStatus        0xFFFF

#define TRUE                           1
#define FALSE                          0
#define NULL                           ( void * ) 0


#define TimeEventArraySize             8

typedef struct {
                 TimeEventElementType  TimeEventArray [ TimeEventArraySize ] ;
                 uint32_t              TimeEventStatustArray [ TimeEventArraySize ] ;
                 uint32_t              TimeTickCounter ;
                 uint8_t               TimeTickEvent ;
               } TimeEventModuleInstanceType ;



static TimeEventModuleInstanceType TimeEventModuleInstance ;


void SysTimerTickEvent ( void )
{
  /*----------------------------------------------------------------*/
  TimeEventModuleInstance . TimeTickEvent = TRUE ;
} /* SysTimerTickEvent */


PtrTimeEventElementType AttachTimer ( TimeEventServiceProcT Service ,
                                      uint32_t              DelayTime )
{
  uint8_t Loop ;
  /*----------------------------------------------------------------*/
  if ( ! DelayTime )
    return NULL ;
  for ( Loop = 0 ; Loop < TimeEventArraySize ; Loop ++ )
  {
    if ( TimeEventModuleInstance . TimeEventStatustArray [ Loop ] == FreeElementStatus )
    {
      TimeEventModuleInstance . TimeEventStatustArray [ Loop ] = SingleCallServiceStatus ;
      TimeEventModuleInstance . TimeEventArray [ Loop ] . TimeService = Service ;
      TimeEventModuleInstance . TimeEventArray [ Loop ] . TimeInterval =
                TimeEventModuleInstance . TimeTickCounter + DelayTime ;
      return ( & TimeEventModuleInstance . TimeEventArray [ Loop ] ) ;
    } /* if */ ;
  } /* for */ ;
  return NULL ;
} /* AttachTimer */


PtrTimeEventElementType AttachPermanentTimer ( TimeEventServiceProcT Service ,
                                               uint32_t              DelayTime )
{
  uint8_t Loop ;
  /*----------------------------------------------------------------*/
  if ( ! DelayTime )
    return NULL ;
  for ( Loop = 0 ; Loop < TimeEventArraySize ; Loop ++ )
  {
    if ( TimeEventModuleInstance . TimeEventStatustArray [ Loop ] == FreeElementStatus )
    {
      TimeEventModuleInstance . TimeEventStatustArray [ Loop ] = DelayTime ;
      TimeEventModuleInstance . TimeEventArray [ Loop ] . TimeService = Service ;
      TimeEventModuleInstance . TimeEventArray [ Loop ] . TimeInterval =
                TimeEventModuleInstance . TimeTickCounter + DelayTime ;
      return ( & TimeEventModuleInstance . TimeEventArray [ Loop ] ) ;
    } /* if */ ;
  } /* for */ ;
  return NULL ;
} /* AttachPermanentTimer */


void DetachTimer ( PtrTimeEventElementType ServiceHandle )
{
  uint8_t Loop ;
  /*----------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < TimeEventArraySize ; Loop ++ )
  {
    if ( TimeEventModuleInstance . TimeEventStatustArray [ Loop ] != FreeElementStatus )
    {
      if ( ServiceHandle == & TimeEventModuleInstance . TimeEventArray [ Loop ] )
      {
        TimeEventModuleInstance . TimeEventStatustArray [ Loop ] = FreeElementStatus ;
        return ;
      } /* if */ ;
    } /* if */ ;
  } /* for */ ;
} /* DetachTimer */


void SysTimerPool ( void )
{
  uint8_t Loop ;
  /*----------------------------------------------------------------*/
  if ( TimeEventModuleInstance . TimeTickEvent )
  {
    TimeEventModuleInstance . TimeTickCounter ++ ;
    TimeEventModuleInstance . TimeTickEvent = FALSE ;
    for ( Loop = 0 ; Loop < TimeEventArraySize ; Loop ++ )
    {
      if ( TimeEventModuleInstance . TimeEventStatustArray [ Loop ] != FreeElementStatus )
      {
        if ( TimeEventModuleInstance . TimeTickCounter ==
             TimeEventModuleInstance . TimeEventArray [ Loop ] . TimeInterval )
        {
          if ( TimeEventModuleInstance . TimeEventArray [ Loop ] . TimeService )
          {
            TimeEventModuleInstance . TimeEventArray [ Loop ] . TimeService ( ) ;
          } /* if */ ;
          if ( TimeEventModuleInstance . TimeEventStatustArray [ Loop ] == SingleCallServiceStatus )
          {
            TimeEventModuleInstance . TimeEventStatustArray [ Loop ] = FreeElementStatus ;
          } /* if ... */
          else
          {
            TimeEventModuleInstance . TimeEventArray [ Loop ] . TimeInterval =
                TimeEventModuleInstance . TimeTickCounter +
                TimeEventModuleInstance . TimeEventStatustArray [ Loop ] ;
          } /* if ... else */ ;
        } /* if */ ;
      } /* if */ ;
    } /* for */ ;
  } /* if */ ;
} /* SysTimerPool */


unsigned char NumberOfEvents ( void )
{
  uint8_t Loop ;
  uint8_t Pcs ;
  /*----------------------------------------------------------------*/
  Pcs = 0 ;
  for ( Loop = 0 ; Loop < TimeEventArraySize ; Loop ++ )
  {
    if ( TimeEventModuleInstance . TimeEventStatustArray [ Loop ] != FreeElementStatus )
    {
      Pcs ++ ;
    } /* if */ ;
  } /* for */ ;
  return Pcs ;
} /* NumberOfEvents */


void SoftTimeEventInit ( void )
{
  unsigned char Loop ;
  /*----------------------------------------------------------------*/
  TimeEventModuleInstance . TimeTickCounter = 0 ;
  TimeEventModuleInstance . TimeTickEvent = FALSE ;
  for ( Loop = 0 ; Loop < TimeEventArraySize ; Loop ++ )
  {
    TimeEventModuleInstance . TimeEventStatustArray [ Loop ] = FreeElementStatus ;
  } /* for */ ;
} /* SoftTimeEventInit */


